@@ -7,6 +7,7 @@
 import java.awt.*;
 import java.awt.event.*;
 import java.io.*;
+import java.nio.file.*;
 import java.util.*;
 import javax.swing.*;
 import java.awt.datatransfer.*;
@@ -20,6 +21,9 @@ public class CDFDump extends JFrame
 				MouseListener, FocusListener, Runnable {
 
     static final long serialVersionUID = 1L;
+    static int macLib = 0;
+    static String cdfbase = null;
+    static String libcdfPath = null;
 
     JFrame parentDriver;
 /** The dialog for user interface. */
@@ -225,8 +229,15 @@ public class CDFDump extends JFrame
     }
 
     CDFDump(JFrame parentDriver, String cdfbase, boolean modal) {
-        if (cdfbase != null) command = "\"" + cdfbase + separator + "bin" + 
-				       separator + "cdfdump\" ";
+        if (cdfbase == null) {
+          cdfbase = System.getProperty("CDF_BASE");
+          if (cdfbase == null)
+            cdfbase = CDFToolsUtil.getenv("CDF_BASE");
+	}
+	if (cdfbase != null)
+	  command = cdfbase + separator + "bin" + separator + "cdfdump ";
+	else
+	  command = "cdfdump ";
         dialog = new JDialog(this, modal);
 	this.parentDriver = parentDriver;
         this.modal = modal;
@@ -239,11 +250,14 @@ public class CDFDump extends JFrame
 
         WindowListener l = new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
+	      if (macLib == 1) new File(libcdfPath).delete();
 	      if (frommain == 0) {
                 if (modal) dialog.dispose();
                 else dialog.setVisible(false);
 		if (parentDriver != null) parentDriver.setVisible(true);
-	      } else {System.exit(1);}
+	      } else {
+		System.exit(1);
+	      }
 	   }
         };
         dialog.addWindowListener(l);
@@ -356,7 +370,6 @@ public class CDFDump extends JFrame
         cdf.setFont(boldFont);
         OUTF.setFont(boldFont);
         outf.setFont(boldFont);
-//	dialog.getContentPane().add(panel1,BorderLayout.NORTH);
 	panelx.add(panel1,BorderLayout.NORTH);
 	
         tpanel1.add(DUMP,BorderLayout.WEST);
@@ -367,24 +380,15 @@ public class CDFDump extends JFrame
 
         makeJButton(panel2n, Box.createRigidArea(vpad5), 0, 0, 1, 1, 0.0, 0.0);
 	makeJButton(panel2n, abc, 0, 1, 1, 1, 0.0, 0.0);
-//        makeJButton(panel2n, Box.createRigidArea(hpad5), 1, 1, 1, 1, 0.0, 0.0);
         makeJButton(panel2n, qec, 2, 1, 1, 1, 0.0, 0.0);
-//        makeJButton(panel2n, Box.createRigidArea(hpad5), 3, 1, 1, 1, 0.0, 0.0);
 	makeJButton(panel2n, ufc, 4, 1, 1, 1, 0.0, 0.0);
-//        makeJButton(panel2n, Box.createRigidArea(hpad5), 5, 1, 1, 1, 0.0, 0.0);
 	makeJButton(panel2n, crc, 6, 1, 1, 1, 0.0, 0.0);
-//        makeJButton(panel2n, Box.createRigidArea(hpad5), 7, 1, 1, 1, 0.0, 0.0);
 
-//        makeJButton(panel2n, Box.createRigidArea(vpad5), 0, 2, 1, 1, 0.0, 0.0);
         makeJButton(panel2n, dtc, 0, 2, 1, 1, 1.0, 1.0);
-//	makeJButton(panel2n, Box.createRigidArea(hpad5), 1, 2, 1, 1, 0.0, 0.0);
 
-//        makeJButton(panel2n, Box.createRigidArea(vpad5), 0, 3, 1, 1, 0.0, 0.0);
         makeJButton(panel2n, STYLE, 0, 3, 1, 1, 1.0, 1.0);
         makeJButton(panel2n, stylePanel, 1, 3, 1, 1, 1.0, 1.0);
-//	makeJButton(panel2n, Box.createRigidArea(hpad5), 2, 3, 1, 1, 0.0, 0.0);
 
-//        makeJButton(panel2n, Box.createRigidArea(vpad5), 0, 4, 1, 1, 0.0, 0.0);
         makeJButton(panel2n, PRECISION, 0, 4, 1, 1, 1.0, 1.0);
         makeJButton(panel2n, precision, 1, 4, 1, 1, 1.0, 1.0);
 	makeJButton(panel2n, Box.createRigidArea(hpad5), 2, 4, 1, 1, 0.0, 0.0);
@@ -433,7 +437,6 @@ public class CDFDump extends JFrame
         RECS.setFont(boldFont);
         recs.setFont(boldFont);
 
-//        dialog.getContentPane().add(panel2,BorderLayout.CENTER);
         panelx.add(panel2,BorderLayout.CENTER);
 
         // Panel 3 (buttons and text area)
@@ -465,10 +468,8 @@ public class CDFDump extends JFrame
                 }
         };
         scroller.getViewport().add(info);
-//        panel3.add(scroller, BorderLayout.SOUTH);
   	info.setVisible(true);
 
-//  	dialog.getContentPane().add(panel3, BorderLayout.SOUTH);
   	panelx.add(panel3, BorderLayout.SOUTH);
 
         dialog.getContentPane().add(panelx, BorderLayout.NORTH);
@@ -506,11 +507,14 @@ public class CDFDump extends JFrame
     public void actionPerformed(ActionEvent evt) {
 	Object obj = evt.getSource();
 	if (obj == quitb || obj == menuFileExit) { // for "Quit" or "Exit"
+             if (macLib == 1) new File(libcdfPath).delete();
 	     if (frommain == 0) {
                 if (modal) dialog.dispose();
                 else dialog.setVisible(false);
 		if (parentDriver != null) parentDriver.setVisible(true);
-	     } else {System.exit(1);}
+	     } else {
+		System.exit(1);
+	     }
 	}
 	else if (obj == slt1 || obj == slt2) { // for "Select 1|2" button
              if (obj == slt1) { // for "Select1" button
@@ -825,54 +829,64 @@ public class CDFDump extends JFrame
     private void runit(String[] command) {
     // private void runit(String command) {
 	try {
-	      if (command[1] != null)
-                process = runtime.exec(command);
-              else
-                process = runtime.exec(command[0]);
+           String osName = System.getProperty("os.name");
+           String cwd = System.getProperty("user.dir");
+           libcdfPath = cwd + "/libcdf.dylib";
+           if (osName.toLowerCase().indexOf("mac") != -1) {
+               File libcdf = new File(libcdfPath);
+               if (libcdf.exists()) {
+                   macLib = 0;
+               } else {
+                 if (Files.isWritable(Paths.get(cwd))) {
+                   try {
+                     cdfbase = System.getProperty("CDF_BASE");
+                     if (cdfbase == null)
+                         cdfbase = CDFToolsUtil.getenv("CDF_BASE");
+                     Files.copy(Paths.get(cdfbase+"/lib/libcdf.dylib"),
+                                Paths.get(libcdfPath),
+                                StandardCopyOption.REPLACE_EXISTING);
+                     macLib = 1;
+                   } catch (Exception ex) {
+                     System.out.println ("**** Error for copying... "+ex);
+                   }
+                 }
+               }
+           } 
 
-              info.setText("");
-/*
-              if (hlp == 0 && abcX == 0) {
-                if (!filespec.equals("") && !filespec.trim().equals("")) {                 
-                  info.setText("Program is submitted...\n\n");
-		  info.append("A dump file: " + xxx.substring(8).trim() + 
-				"  is to be created.\n");
-                  if (iba == 1) return;
-                } else {
-                  info.setText("");
-                  info.append("Program is running...\n");
-                }
-              }
-*/
-              Dimension dialogSize = dialog.getSize();
-              ((JComponent)(dialog.getContentPane())).paintImmediately(
+	   if (command[1] != null)
+                process = runtime.exec(command);
+           else
+                process = runtime.exec(command[0]);
+           info.setText("");
+           Dimension dialogSize = dialog.getSize();
+           ((JComponent)(dialog.getContentPane())).paintImmediately(
                                             0, 0, dialogSize.width, dialogSize.height);
-              BufferedReader inStream = null;
-              String line = null;
-              inStream = new BufferedReader(new
-                                      InputStreamReader(process.getInputStream()), 8192);
-	      boolean first = true;	
-              while ((line = inStream.readLine()) != null) {
+           BufferedReader inStream = null;
+           InputStreamReader inReader = new InputStreamReader(process.getInputStream(), "UTF-8");
+           String line = null;
+           inStream = new BufferedReader(inReader, 8192);
+	   boolean first = true;	
+           while ((line = inStream.readLine()) != null) {
 		if (first) {
 			info.setText("");
 			first =false;
 		}
                 info.append(line+"\n");
-	      }
-              process.waitFor();
-	      info.setCaretPosition(0);
-              scroller.getViewport().setView(info);
-              scroller.revalidate();
-              scroller.repaint();
+	   }
+           process.waitFor();
+	   info.setCaretPosition(0);
+           scroller.getViewport().setView(info);
+           scroller.revalidate();
+           scroller.repaint();
 
-              if (iba == 1) {
+           if (iba == 1) {
                 String outofpgm = info.getText().toLowerCase().trim();
                 if (outofpgm.indexOf("error") != -1 || 
                     outofpgm.indexOf("fail") != -1)
                   info.append("ERROR: Dump file is not created\n"); 
                 else
                   info.append("Done!\n\n");
-	      }
+	   }
         } catch (Exception e) {
               info.setText("error executing: "+e);
 	      Toolkit.getDefaultToolkit().beep();
@@ -884,9 +898,7 @@ public class CDFDump extends JFrame
  */
 
     private void doFileChooser (String title, JTextField textField, int theone, SimpleFileFilter filter) {
-//        JFileChooser fileChooser = new JFileChooser();
 	ExampleFileView fileView = new ExampleFileView();
-//        fileChooser.setCurrentDirectory(new File(currentdir));
         fileChooser.setDialogTitle(title);
         if (filter != null) {
 	   fileChooser.addChoosableFileFilter(filter);
@@ -916,7 +928,6 @@ public class CDFDump extends JFrame
               textField.setText(myfile);
            }
         } else {
-//                JOptionPane.showMessageDialog(dialog, "No file chosen");
         }
     }
 
@@ -924,15 +935,15 @@ public class CDFDump extends JFrame
  */
 
     static public void main(String[] args) {
-
-        String cdfbase = System.getProperty("CDF_BASE");
+System.err.println("***** cdfdump");
+        cdfbase = System.getProperty("CDF_BASE");
         if (cdfbase == null)
             cdfbase = CDFToolsUtil.getenv("CDF_BASE");
-
 	CDFDump thispgm = new CDFDump(null, cdfbase);
 	thispgm.fileChooser.setCurrentDirectory(new File(thispgm.currentdir));
 	thispgm.frommain = 1;
 	thispgm.run();
+
     }
 
 }
