@@ -1,4 +1,4 @@
-// $Id: EditAttrEntryDialog.java,v 1.1.1.1 2019/10/16 16:22:18 mhliu Exp $
+// $Id: EditAttrEntryDialog.java,v 1.2 2021/08/13 13:48:14 mhliu Exp $
 /******************************************************************************
 * Copyright 1996-2014 United States Government as represented by the
 * Administrator of the National Aeronautics and Space Administration.
@@ -249,7 +249,12 @@ public class EditAttrEntryDialog extends JDialog implements CDFConstants, Action
               String tmp = (String)tfva.get();
               tmp = tmp.substring(1, tmp.length()-1);
               StringBuffer tmpBuffer = new StringBuffer();
-              byte[] tmpData = tmp.getBytes();
+              byte[] tmpData;
+              try {
+                tmpData = tmp.getBytes("UTF-8");
+              } catch (Exception ee) {
+                tmpData = tmp.getBytes();
+              }
               for (int i = 0; i < Array.getLength(tmpData); i++) {
                 if (i > 0) tmpBuffer.append(",");
                 int xx = tmpData[i];
@@ -260,24 +265,25 @@ public class EditAttrEntryDialog extends JDialog implements CDFConstants, Action
               tfva.set(tmpBuffer.toString());
 		outData = (String) tfva.get();
             } else {
-              if ((dataType == CDF_BYTE || dataType == CDF_INT1 || dataType == CDF_UINT1) &&
+              if ((dataType == CDF_BYTE || dataType == CDF_INT1 ||
+                   dataType == CDF_UINT1) &&
                   (dt == CDF_CHAR || dt == CDF_CHAR)) {
                 String tmp = ((String)tfva.get()).trim();
                 StringBuffer newStr = new StringBuffer();
                 StringTokenizer st = new StringTokenizer(tmp, ",");
                 if (st.countTokens() == -1) {
                   int dataValue = Integer.parseInt(tmp);
-                  if (dataType == CDF_UINT1) dataValue = dataValue < 0 ?
-                                                         dataValue-2*Byte.MIN_VALUE : 
-							 dataValue;
+                  if (dataType == CDF_UINT1)
+                    dataValue = (dataValue < 0 ? dataValue-2*Byte.MIN_VALUE : 
+					         dataValue);
                   newStr.append(dataValue);
                 } else {
                    while (st.hasMoreTokens()) {
                      String str = ((String)st.nextToken()).trim();
                      int dataValue = Integer.parseInt(str);
-                     if (dataType == CDF_UINT1) dataValue = dataValue < 0 ?
-                                                            dataValue-2*Byte.MIN_VALUE : 
-							    dataValue;
+                     if (dataType == CDF_UINT1)
+                       dataValue = (dataValue < 0 ? dataValue-2*Byte.MIN_VALUE : 
+						    dataValue);
                      newStr.append(dataValue);
                    }
                 }
