@@ -271,6 +271,7 @@ public interface CDFConstants {
     public static final long TT2000_CDF_MAYNEEDUPDATE     = 1010;
     public static final long BLOCKINGFACTOR_TOO_SMALL     = 1011;
     public static final long BLOCKINGFACTOR_TOO_SMALL2    = 1012;
+    public static final long STRING_NOT_UTF8_ENCODING     = 1013;
 
     public static final long CDF_OK                       = 0;
 
@@ -415,6 +416,7 @@ public interface CDFConstants {
     public static final long FUNCTION_NOT_SUPPORTED       = -2236;
     public static final long TRY_TO_READ_NONSTRING_DATA   = -2237;
     public static final long BAD_NUM_STRINGS              = -2238;
+    public static final long CANNOT_CONVERT_WIDECHAR      = -2239;
 
 
     /*----------------------------------------------------------------*/
