@@ -1,4 +1,4 @@
-//$Id: CDFData.java,v 1.2 2020/02/18 17:28:59 mhliu Exp $
+//$Id: CDFData.java,v 1.4 2022/01/03 16:27:41 mhliu Exp $
 /*
  * Copyright 1996-2014 United States Government as represented by the
  * Administrator of the National Aeronautics and Space Administration.
@@ -634,9 +634,22 @@ synchronized(v) {
     private void arrayify() throws CDFException {
 	if (nDims <= 1 || numValues == 1) {
 	    if ((dataType == CDF_CHAR) || (dataType == CDF_UCHAR)) {
-		if (nDims == 0)
-		    _dataArray = new String((byte [])_data);
-		else {
+		if (nDims == 0) {
+                    int i, iend;
+                    iend = ((byte [])_data).length;
+                    for (i = 0; i < ((byte [])_data).length; ++i) {
+                      if (((byte [])_data)[i] == 0) {
+                         iend = i;
+                         break;
+                      }
+		    }
+//		    _dataArray = new String((byte [])_data);
+		    try {
+		      _dataArray = new String((byte [])_data, 0, iend, "UTF-8");
+		    } catch (Exception ee) {
+		      _dataArray = new String((byte [])_data, 0, iend);
+		    }  
+		}		else {
 		    buildDataArray();
 		    int n = 0;
 		    byte [] aByteArray = new byte[(int)numElements];
@@ -650,7 +663,13 @@ synchronized(v) {
                             break;
                           }
                         }
-			Array.set(_dataArray, i, new String(aByteArray, 0, iend));
+//			Array.set(_dataArray, i, new String(aByteArray, 0, iend));
+			try {
+			  Array.set(_dataArray, i, new String(aByteArray, 0, iend,
+                                                              "UTF-8"));
+                        } catch (Exception ee) {
+			  Array.set(_dataArray, i, new String(aByteArray, 0, iend));
+                        }
 			n += numElements;
 		    }
 		}
@@ -729,7 +748,13 @@ synchronized(v) {
                                 break;
                               }
                             }
-			    ((String [])aRow)[j] = new String(aByteArray, 0, iend);
+//			    ((String [])aRow)[j] = new String(aByteArray, 0, iend);
+			    try {
+			      ((String [])aRow)[j] = new String(aByteArray, 0, iend,
+							        "UTF-8");
+			    } catch (Exception ee) {
+			      ((String [])aRow)[j] = new String(aByteArray, 0, iend);
+			    }
 			    n += numElements;
 			}
 			break;
@@ -814,10 +839,12 @@ synchronized(v) {
 		    for (int i=0; i<numElements; i++)	// fill with spaces or 0
 			((byte [])_data)[i] = 0;
                     if (((String)_dataArray).length() != 0) {
-                      lenx = ((String)_dataArray).length();
-		      System.arraycopy(((String)_dataArray).getBytes(),
-				       0, _data, 0,
-				       (int)((lenx>numElements)?numElements:lenx));
+                      String oneStr = (String)_dataArray;
+                      byte[] bytes = oneStr.getBytes();
+                      lenx = bytes.length;
+		      System.arraycopy(bytes, 0, _data, 0,
+				       (int)((lenx>numElements)?numElements:
+                                                                lenx));
 		    }
 		  } else {
 		    buildDataObject();
@@ -825,10 +852,12 @@ synchronized(v) {
                         for (int j=0; j<numElements; j++)
                             ((byte [])_data)[n+j] = 0;
                         if (((String [])_dataArray)[i].length() != 0) {
-                          lenx = ((String [])_dataArray)[i].length();
-			  System.arraycopy(((String [])_dataArray)[i].getBytes(),
-					   0, _data, n, 
-					   (int)((lenx>numElements)?numElements:lenx));
+                          String oneStr = ((String [])_dataArray)[i];  
+                          byte[] bytes = oneStr.getBytes();   
+                          lenx = bytes.length;
+		          System.arraycopy(bytes, 0, _data, n,
+				           (int)((lenx>numElements)?numElements:
+                                                                    lenx));
 			}
 			n += numElements;
 		    }
@@ -883,7 +912,8 @@ synchronized(v) {
 			    // subsequent subarray
 			    for (int j=i+1;j<(int)nDims-1;j++) {
 				cIndex[j] = 0;
-				subArrays[j] = Array.get(subArrays[j-1],cIndex[j]);
+				subArrays[j] = Array.get(subArrays[j-1],
+                                                         cIndex[j]);
 			    }
 			    break;
 			    
@@ -898,17 +928,20 @@ synchronized(v) {
 		    switch ((int)_myVar.getDataType()) {
 		    case (int)CDF_CHAR:
 		    case (int)CDF_UCHAR:
-			aRow = (String [])Array.get(subArrays[(int)nDims - 2], i);
+			aRow = (String [])Array.get(subArrays[(int)nDims - 2],
+                                                    i);
 			for (int j=0; j<dimSizes[nDims - 1]; j++) {
-                            for (int k=0; k<numElements; k++)
-                                ((byte [])_data)[n+k] = 0;
-			    if (((String [])aRow)[j] != null) {
-                                int lenx = ((String [])aRow)[j].length();
-				System.arraycopy(((String [])aRow)[j].getBytes(),0,
-						 _data,n,
-                                                 (int)((lenx>numElements)?numElements:lenx));
-			    }
-			    n += numElements;
+                          for (int k=0; k<numElements; k++)
+                             ((byte [])_data)[n+k] = 0;
+			  if (((String [])aRow)[j] != null) {
+                             String oneStr = ((String [])aRow)[j];
+			  byte[] bytes = oneStr.getBytes();
+			  int lenx = bytes.length;
+			  System.arraycopy(bytes, 0, _data, n,
+                                           (int)((lenx>numElements)?numElements:
+                                                                    lenx));
+			  }
+			  n += numElements;
 			}
 			break;
 		    case (int)CDF_BYTE:
@@ -1114,14 +1147,13 @@ synchronized(v) {
 	    _data = new byte [(int)numValues];
             for (int i=0; i<(int)numValues; i++) ((byte [])_data)[i] = 0;
 	    if (padValue == null) {
-              if (dataTypeX == (int)CDF_CHAR)
-                ((byte [])_data)[0] = 
-                      (((new Character(DEFAULT_CHAR_PADVALUE))
-                                      .toString()).getBytes())[0];
-	      else
-		((byte [])_data)[0] = 
-		      (((new Character(DEFAULT_UCHAR_PADVALUE))
-				      .toString()).getBytes())[0];
+              if (dataTypeX == (int)CDF_CHAR) {
+                ((byte [])_data)[0] = (((new Character(DEFAULT_CHAR_PADVALUE))
+                                        .toString()).getBytes())[0];
+	      } else {
+		((byte [])_data)[0] = (((new Character(DEFAULT_UCHAR_PADVALUE))
+		  		        .toString()).getBytes())[0];
+              }
               for (int i=1; i<numValues; i++)
                   ((byte [])_data)[i] = 0; 
 	    } else {
