@@ -1,4 +1,4 @@
-// $Id: CDFTools.java,v 1.2 2020/01/31 14:47:47 mhliu Exp $
+// $Id: CDFTools.java,v 1.2 2021/10/01 14:51:33 mhliu Exp $
 /*
  * Copyright 1996-2014 United States Government as represented by the
  * Administrator of the National Aeronautics and Space Administration.
@@ -19,7 +19,7 @@ import gsfc.nssdc.cdf.util.*;
  * Created: Tue Nov 24 16:14:50 1998
  *
  * @author Phil Williams
- * @version $Id: CDFTools.java,v 1.2 2020/01/31 14:47:47 mhliu Exp $
+ * @version $Id: CDFTools.java,v 1.2 2021/10/01 14:51:33 mhliu Exp $
  */
 
 public class CDFTools implements CDFConstants {
@@ -794,7 +794,7 @@ public class CDFTools implements CDFConstants {
 
 	// Echo any errors
 	BufferedReader inStream = 
-	    new BufferedReader(new InputStreamReader(p.getInputStream()));
+	    new BufferedReader(new InputStreamReader(p.getInputStream(), "UTF-8"));
 	String line;
 	while ((line = inStream.readLine()) != null) {
 	    if (err instanceof PrintStream)
@@ -1240,7 +1240,7 @@ public class CDFTools implements CDFConstants {
 
 	// Echo any errors
 	BufferedReader inStream = 
-	    new BufferedReader(new InputStreamReader(p.getInputStream()));
+	    new BufferedReader(new InputStreamReader(p.getInputStream(), "UTF-8"));
 	String line;
 	while ((line = inStream.readLine()) != null) {
 	    if (err instanceof PrintStream)
