@@ -1,4 +1,4 @@
-// $Id: CDFUtils.java,v 1.2 2020/03/23 19:17:19 mhliu Exp $
+// $Id: CDFUtils.java,v 1.5 2022/01/26 17:24:18 mhliu Exp $
 /*
  * Copyright 1996-2014 United States Government as represented by the
  * Administrator of the National Aeronautics and Space Administration.
@@ -178,7 +178,7 @@ public class CDFUtils implements CDFConstants {
      */
     public static void printData (Object data) {
 
-	CDFUtils.printData(data, 0, true);
+	CDFUtils.printData(data, 0, true, "");
     }
 
     /**
@@ -189,22 +189,58 @@ public class CDFUtils implements CDFConstants {
      *  @param data the data to be printed
      *  @param which the Epoch data type data indicator
      *               1 if CDF_EPOCH, 2 if CDF_EPOCH16 or 3 if CDF_TIME_TT2000
+     *               0 Otherwise
      *  @param iso8601 the ISO 8601 indicator for EPOCH data
      *
      */
     public static void printData (Object data, int which, boolean iso8601) {
 
-        CDFUtils.printData(data, 0, true, "");
+        CDFUtils.printData(data, which, iso8601, "");
+    }
+/*
+    public static void printMultiData (Object data) {
+        CDFUtils.printMultiData(data, 0, true, "");
     }
 
+    public static void printMultiData (Object data, int which, boolean iso8601) {
+        CDFUtils.printMultiData(data, which, iso8601, "");
+    }
+
+    public static void printMultiData (Object data, int which, boolean iso8601,
+                                       String addZ) {
+
+        if (data == null) return;
+        int nDims = CDFUtils.getNumDimensions(data);
+        if (nDims < 2) {
+          printData (data, which, iso8601, addZ);
+          System.out.println("");
+        } else {
+          int arrayLength = ((Object[])data).length;
+          int i;
+          Object subObj;
+          for (i = 0; i < arrayLength; ++i) {
+            subObj = ((Object[])data)[i];
+            nDims = CDFUtils.getNumDimensions(subObj);
+            if (nDims > 1)
+              printMultiData (subObj, which, iso8601, addZ);
+            else {
+              printData (subObj, which, iso8601, addZ);
+              System.out.println("");
+            }
+          }
+        }
+
+    }
+*/
     /**
      *  Prints the value of the given data on the screen.  Data can be a
-     *  java primitive data type, Java Object (non-array), or
-     *  1-dimensional array of primitive Java data type.
+     *  java primitive data type, Java Object (non-array), or one or
+     *  multi-dimensional array of primitive Java data type.
      *
      *  @param data the data to be printed
      *  @param which the Epoch data type data indicator
      *               1 if CDF_EPOCH, 2 if CDF_EPOCH16 or 3 if CDF_TIME_TT2000
+     *               0 Otherwise
      *  @param iso8601 the ISO 8601 indicator for EPOCH data
      *  @param addZ Added the passed string to the end, (likely a "Z", to
      *              all CDF epoch data types, not just CDF_TIME_TT2000)
@@ -213,10 +249,23 @@ public class CDFUtils implements CDFConstants {
     public static void printData (Object data, int which, boolean iso8601,
                                   String addZ) {
 
-        int  i, arrayLength, jj = 1;
-        String signature = CDFUtils.getSignature(data);
-	if (which == 2) jj = 2;
-        if (signature.charAt(0) == '[') {
+        if (data == null) return;
+        int nDims = CDFUtils.getNumDimensions(data);
+        if (nDims > 1) {
+          int arrayLength = ((Object[])data).length;
+          int i;
+          Object subObj;
+          for (i = 0; i < arrayLength; ++i) {
+            subObj = ((Object[])data)[i];
+            printData (subObj, which, iso8601, addZ);
+//            System.out.println("");
+          }
+        } else {
+          /* Scalar or 1-D data */
+          int  i, arrayLength, jj = 1;
+          String signature = CDFUtils.getSignature(data);
+          if (which == 2) jj = 2;
+          if (signature.charAt(0) == '[') {
             arrayLength = Array.getLength(data);
             for (i=0; i < arrayLength; i=i+jj) {
                  if (signature.charAt(1) == 'B')
@@ -261,22 +310,30 @@ public class CDFUtils implements CDFConstants {
                      System.out.print (Array.getDouble(data,i)+
                                        (i==(arrayLength-jj)?"":" "));
                  } else {
-                     if (i>0) System.out.print(" \"");
-                     System.out.print (Array.get((String[])data, i));
-                     if (i<(arrayLength-1)) System.out.print("\" ");
+                     System.out.print("\"");
+                     System.out.print(Array.get((String[])data, i));
+                     System.out.print("\" ");
+//                     if (i<(arrayLength-1)) System.out.print("\" ");
                  }
+//                 System.out.println("");
             }
+            System.out.println("");
+          }
+          else {
+	    if (which == 1) 
+              System.out.println ((!iso8601)?
+                                Epoch.encode(((Double)data).doubleValue()):
+                                Epoch.encode4(((Double)data).doubleValue())+addZ);
+            else if (which == 3)
+              System.out.println (CDFTT2000.encode(((Long)data).longValue())+addZ);
+            else {
+              if (data instanceof String)
+                System.out.println ("\""+data+"\"");
+              else
+                System.out.println (data);
+            }
+	  }
         }
-        else {
-	   if (which == 1) 
-             System.out.print ((!iso8601)?
-                               Epoch.encode(((Double)data).doubleValue()):
-                               Epoch.encode4(((Double)data).doubleValue())+addZ);
-           else if (which == 3)
-             System.out.print (CDFTT2000.encode(((Long)data).longValue())+addZ);
-           else
-             System.out.print (data);
-	}
     }
 
 
@@ -288,6 +345,7 @@ public class CDFUtils implements CDFConstants {
      *  @param data the data to be printed
      *  @param which the Epoch data type data indicator
      *               1 if CDF_EPOCH, 2 if CDF_EPOCH16 or 3 if CDF_TIME_TT2000
+     *               0 Otherwise
      *
      */
     public static void printData (Object data, int which) {
@@ -335,6 +393,7 @@ public class CDFUtils implements CDFConstants {
      *                   of the object/data is printed as a text-output stream
      *  @param which the Epoch data type data indicator
      *               1 if CDF_EPOCH, 2 if CDF_EPOCH16 or 3 if CDF_TIME_TT2000
+     *               0 Otherwise
      */
     public static void printData (Object data, PrintWriter outWriter,
 				  int which) {
@@ -352,6 +411,7 @@ public class CDFUtils implements CDFConstants {
      *                   of the object/data is printed as a text-output stream
      *  @param which the Epoch data type data indicator
      *               1 if CDF_EPOCH, 2 if CDF_EPOCH16 or 3 if CDF_TIME_TT2000
+     *               0 Otherwise
      *  @param iso8601 the ISO 8601 indicator for EPOCH data
      */
     public static void printData (Object data, PrintWriter outWriter,
@@ -433,6 +493,7 @@ public class CDFUtils implements CDFConstants {
              outWriter.print (CDFTT2000.encode(((Long)data).longValue())+addZ);
            else
 	     outWriter.print (data);
+
 	}
     }
 
@@ -462,6 +523,7 @@ public class CDFUtils implements CDFConstants {
      *  @param format the format specifically for the floating point data
      *  @param which the Epoch data type data indicator
      *               1 if CDF_EPOCH, 2 if CDF_EPOCH16 or 3 if CDF_TIME_TT2000
+     *               0 otherwise
      *  @param iso8601 the ISO 8601 indicator for EPOCH data
      *
      */
@@ -506,29 +568,21 @@ public class CDFUtils implements CDFConstants {
             arrayLength = Array.getLength(data);
             for (i=0; i < arrayLength; i=i+jj) {
                  if (signature.charAt(1) == 'B') {
-//                   System.out.format (jFormat+(i==(arrayLength-jj)?"":" "),
-//                                      Array.getByte(data,i));
                      System.out.print (Array.getByte(data,i)+
                                        (i==(arrayLength-jj)?"":" "));
 
                  } else if (signature.charAt(1) == 'S') {
-//                   System.out.format (jFormat+(i==(arrayLength-jj)?"":" "),
-//                                      Array.getShort(data,i));
                      System.out.print (Array.getShort(data,i)+
                                        (i==(arrayLength-jj)?"":" "));
 
 
                  } else if (signature.charAt(1) == 'I') {
-//                   System.out.format (jFormat+(i==(arrayLength-jj)?"":" "),
-//                                      Array.getInt(data,i));
                      System.out.print (Array.getInt(data,i)+
                                        (i==(arrayLength-jj)?"":" "));
 
 
                  } else if (signature.charAt(1) == 'J') {
                      if (which == 0) {
-//                       System.out.format (jFormat+(i==(arrayLength-jj)?"":" "),
-//                                          Array.getLong(data,i));
                          System.out.print (Array.getLong(data,i)+
                                            (i==(arrayLength-jj)?"":" "));
 
@@ -586,8 +640,8 @@ public class CDFUtils implements CDFConstants {
                    CDFUtils.isFloatingPadValue(data) == 0)
                  System.out.format (jFormat,data);
                else
-                 System.out.print (data);
-                 
+                 System.out.println (data);
+
            }
 	}
     }
@@ -602,6 +656,7 @@ public class CDFUtils implements CDFConstants {
      *  @param format the format specifically for the floating point data
      *  @param which the Epoch data type data indicator
      *               1 if CDF_EPOCH, 2 if CDF_EPOCH16 or 3 if CDF_TIME_TT2000
+     *               0 otherwise
      *
      */
     public static void printDataWithFormat (Object data, String format,
@@ -719,20 +774,16 @@ public class CDFUtils implements CDFConstants {
             arrayLength = Array.getLength(data);
             for (i=0; i < arrayLength; i=i+jj) {
                  if (signature.charAt(1) == 'B') {
-//                   outWriter.format (jFormat+" ", Array.getByte(data,i));
                      outWriter.print (Array.getByte(data,i)+" ");
 
                  } else if (signature.charAt(1) == 'S') {
-//                   outWriter.format (jFormat+" ", Array.getShort(data,i));
                      outWriter.print (Array.getShort(data,i)+" ");
 
                  } else if (signature.charAt(1) == 'I') {
-//                   outWriter.format (jFormat+" ", Array.getInt(data,i));
                      outWriter.print (Array.getInt(data,i)+" ");
 
                  } else if (signature.charAt(1) == 'J') {
                      if (which == 0) {
-//                       outWriter.format (jFormat+" ", Array.getLong(data,i));
                          outWriter.print (Array.getLong(data,i)+" ");
                      } else
                        outWriter.print (CDFTT2000.encode(Array.getLong(data,i))+
@@ -825,7 +876,12 @@ public class CDFUtils implements CDFConstants {
           }
           return tmp.toString();
         } else
-          return (String)data;
+//          return (String)data;
+          try {
+            return new String(((String)data).getBytes(), "UTF-8");
+          } catch (Exception ee) {
+            return (String)data;
+          }
     }
 
     /**
@@ -2315,4 +2371,149 @@ public class CDFUtils implements CDFConstants {
         else return 0;
     }
 
+    public static int getNumDimensions(Object obj) {
+    /**
+     * Get the number of dimensions of the given object.
+     * @param obj The given object
+     * @return the number of dimensions
+     */
+
+        String sig = CDFUtils.getSignature(obj);
+        int i, ndims = 0;
+        for (i = 0; i < sig.length(); ++i) {
+          if (sig.charAt(i) != '[') break;
+          ++ndims;
+        }
+        return ndims;
+/*
+        Object tmp = obj;
+        int nDimensions = 0;
+        while (true) {
+          if (tmp instanceof Object[]) {
+            tmp = ((Object[]) tmp)[0];
+            nDimensions++;
+          }
+          else if (tmp.getClass().isArray()) {
+            ++nDimensions;
+            break;
+          } else
+            break;
+        }
+        return nDimensions;
+*/
+    }
+
+    public static long[] getDimensionSizes(Object obj) {
+    /**
+     * Returned the dimensional sizes of the given object.
+     * @param obj The given object
+     * @return dimensiosnal sizes
+     *         null if obj is a scalar
+     */
+
+        int ndims = 0;
+        ndims = getNumDimensions(obj);
+        if (ndims == 0) return null;
+        long[] dims = new long[ndims]; 
+        int i = 0;
+        Object tmp = obj;
+        while (true) {
+          if (tmp instanceof Object[]) {
+            dims[i] = (long) ((Object[]) tmp).length;
+            tmp = ((Object[]) tmp)[0];
+            ++i;
+          }
+          else if (tmp.getClass().isArray()) {
+            dims[i] = (long) Array.getLength(tmp);
+            break;
+          } else
+            break;
+        }
+        return dims;
+    }
+
+    public static byte[] UTF8toUTF16LE(Object utf8) {
+    /**
+     * Convert a string or UTF-8 encoded byte array to a UTF-16LE encoded byte
+     * array.
+     * @param utf8 A string or a UTF8 encoded byte array
+     * @return UTF-16LE encoded byte array
+     *         null if an error occurs while conversion
+     */
+
+        String signature = CDFUtils.getSignature (utf8);
+        try {
+          if (signature.charAt(0) == '[') {
+            return (new String((byte[])utf8, "UTF-8")).getBytes("UTF-16LE");
+          } else {
+            return ((String)utf8).getBytes("UTF-16LE");
+          }
+        } catch (Exception ee) {
+          return null;
+        }
+    }
+
+    public static byte[] UTF16LEtoUTF8(Object utf16) {
+    /**
+     * Convert a string or a UTF-16LE encoded byte array to a UTF-8 encoded
+     * byte array. 
+     * @param utf16 A string or a UTF-16LE encoded byte array
+     * @return UTF-8 encoded byte array
+     *         null if an error occurs while conversion
+     */
+
+        String signature = CDFUtils.getSignature (utf16);
+        try {
+          if (signature.charAt(0) == '[') {
+            return (new String((byte[])utf16, "UTF-16LE")).getBytes("UTF-8");
+          } else {
+            return ((String)utf16).getBytes("UTF-8");
+          }
+        } catch (Exception ee) {
+          return null;
+        }
+    }
+
+    public static byte[] UTF8toCp1252(Object utf8) {
+    /**
+     * Convert a string or a UTF-8 encoded byte array to a Windows-1252 encoded
+     * byte array.
+     * @param utf8 A string or UTF8 encoded byte array
+     * @return Windows-1252 encoded byte array
+     *         null if an error occurs while conversion
+     */
+
+        String signature = CDFUtils.getSignature (utf8);
+        try {
+          if (signature.charAt(0) == '[') {
+            return (new String((byte[])utf8, "UTF-8")).getBytes("Windows-1252");
+          } else {
+            return ((String)utf8).getBytes("Windows-1252");
+          }
+        } catch (Exception ee) {
+          return null;
+        }
+    }
+
+    public static byte[] Cp1252toUTF8(Object cp1252) {
+    /**
+     * Convert a string or Windows-1252 encoded byte array to a UTF-8 encoded
+     * byte array.
+     * @param cp1252 A string or a Windows-1252 encoded byte array
+     * @return UTF-8 encoded byte array
+     *         null if an error occurs while conversion
+     */
+
+        String signature = CDFUtils.getSignature (cp1252);
+        try {
+          if (signature.charAt(0) == '[') {
+            return (new String((byte[])cp1252, "Windows-1252")).getBytes("UTF-8");
+          } else {
+            return ((String)cp1252).getBytes("UTF-8");
+          }
+        } catch (Exception ee) {
+          return null;
+        }
+    }
+
 }
