@@ -1,4 +1,4 @@
-// $Id: Epoch.java,v 1.3 2020/06/24 18:30:23 mhliu Exp $
+// $Id: Epoch.java,v 1.4 2021/06/07 16:55:55 mhliu Exp $
 /*
  * Copyright 1996-2014 United States Government as represented by the
  * Administrator of the National Aeronautics and Space Administration.
@@ -10,7 +10,7 @@ package gsfc.nssdc.cdf.util;
 import gsfc.nssdc.cdf.CDFException;
 import java.text.NumberFormat;
 import java.text.DecimalFormat;
-
+import java.lang.reflect.*;
 import java.util.*;
 import java.text.*;
 
@@ -111,43 +111,6 @@ public class Epoch implements gsfc.nssdc.cdf.CDFConstants {
         for (ix = 0; ix < inStrings.length; ++ix)
           dateTimes[ix] = toParse (inStrings[ix]);
         return dateTimes;
-/*
-        String dateTime = inStrings[0].trim();
-        int len = dateTime.length();
-        char aC;
-        if (len == EPOCH_STRING_LEN) { // style 0 or style 3
-          if (dateTime.charAt(11) == ' ') return parse(inStrings);
-          aC = dateTime.charAt(10);
-          if (aC == 'T' || aC == 't' || aC == ' ' ||
-              aC == '/') return parse3(inStrings);
-          else return null;
-        } else if (len == EPOCH4_STRING_LEN) {
-          aC = dateTime.charAt(10);
-          if (aC == 'T' || aC == 't' || aC == ' ' ||
-              aC == '/') return parse4(inStrings);
-          else return null;
-        } else if (len == EPOCH1_STRING_LEN &&
-                   dateTime.matches("\\d+.\\d+")) {
-          return parse1(inStrings);
-        } else if (len == EPOCH2_STRING_LEN &&
-                   dateTime.matches("\\d+")) {
-          return parse2(inStrings);
-//        } else if (len == (EPOCH_STRING_LEN+1) && (dateTime.endsWith("Z") ||
-//                                                   dateTime.endsWith("z")) &&
-//                   dateTime.charAt(11) == ' ') { // style 0 (sort of)
-//          return parse(inStrings);
-        } else if (dateTime.charAt(11) == ' ') {
-          return parse(inStrings);
-        } else if (dateTime.charAt(10) == 'T' || dateTime.charAt(10) == 't' ||
-                   dateTime.charAt(10) == ' ' ||
-                   dateTime.charAt(10) == '/') {
-            if (dateTime.endsWith("Z") || dateTime.endsWith("Z"))
-              return parse3(inStrings);
-            else
-              return parse4(inStrings);
-        } else
-          return null;
-*/
     }
 
     /**
@@ -754,6 +717,61 @@ public class Epoch implements gsfc.nssdc.cdf.CDFConstants {
         return strings;
     }
 
+    /**
+     * 
+     * Converts an EPOCH object into a scalar or array(s) of readable
+     * date/time string(s).
+     * Each string is presented in the following ISO 8601 form: 
+     *
+     * <PRE>
+     *
+     * Format:    yyyymmddThh:mm:ss.ccc
+     * Examples:  19900401T03:05:02.000
+     *            19931010T23:45:49.999
+     *
+     * </PRE>
+     *
+     * @param epoch the epoch object
+     *
+     * @return A string or array(s) of strings representation of the epoch
+     */
+    public static Object toEncode (Object epoch)
+    {
+        if (epoch == null) return null;
+        int nDims = CDFUtils.getNumDimensions(epoch);
+        if (nDims > 1) {
+          Object encodedArray = null;
+          String className = "java.lang.String";
+
+          long[] dimSizes = CDFUtils.getDimensionSizes(epoch);
+          int[] tempSizes = new int[nDims];
+          for (int j = 0; j < nDims; ++j) tempSizes[j] = (int) dimSizes[j];
+          Class arrayClass;
+          try {
+            arrayClass = Class.forName(className);
+            encodedArray = Array.newInstance(arrayClass, tempSizes);
+          } catch (ClassNotFoundException e) {
+            System.out.println("**** error BAD_ARGUMENT");
+            System.exit(0);
+          }
+
+          int arrayLength = ((Object[])epoch).length;
+          int i;
+          Object subObj;
+          for (i = 0; i < arrayLength; ++i) {
+            ((Object[])encodedArray)[i] = toEncode (((Object[])epoch)[i]);
+          }
+          return encodedArray;
+        } else {
+          String sig = CDFUtils.getSignature (epoch);
+          if (sig.charAt(0) != '[') { /* scalar */
+            return toEncode((Double)epoch, 4);
+          } else { /* 1-D */
+            return toEncode((double[])epoch, 4);
+          }
+        }
+    }
+
     /**
      * 
      * Converts an EPOCH value into a readable date/time string.
@@ -1343,7 +1361,6 @@ public class Epoch implements gsfc.nssdc.cdf.CDFConstants {
 	String ePartStr = ePart.toString();
 	char sp = new DecimalFormat().getDecimalFormatSymbols().getDecimalSeparator();
 	appendPart(encoded,
-//		   ePartStr.substring(ePartStr.indexOf(".")+1),
 		   ePartStr.substring(ePartStr.indexOf(sp)+1),		   
 		   width,false);
     }
@@ -1693,7 +1710,7 @@ public class Epoch implements gsfc.nssdc.cdf.CDFConstants {
     {
 	long [] components = new long[7];
 	long jd,i,j,k,l,n;
-	long msec_AD, second_AD, minute_AD, hour_AD, day_AD;
+	double msec_AD, second_AD, minute_AD, hour_AD, day_AD;
 	
 	if (epoch == -1.0E31) {
 	  components[0] = 9999;
@@ -1709,13 +1726,13 @@ public class Epoch implements gsfc.nssdc.cdf.CDFConstants {
 	if (epoch < 0.0) epoch = -epoch;
 	epoch = (MAX_EPOCH_BINARY < epoch ? MAX_EPOCH_BINARY : epoch);
 
-	msec_AD = (long) epoch;
-	second_AD = msec_AD / 1000;
-	minute_AD = second_AD / 60;
-	hour_AD = minute_AD / 60;
-	day_AD = hour_AD / 24;
+	msec_AD = epoch;
+	second_AD = msec_AD / 1000.0;
+	minute_AD = second_AD / 60.0;
+	hour_AD = minute_AD / 60.0;
+	day_AD = hour_AD / 24.0;
 	
-	jd = (long) (1721060 + day_AD);
+	jd = 1721060 + (long) day_AD;
 	l=jd+68569;
 	n=4*l/146097;
 	l=l-(146097*n+3)/4;
@@ -1731,10 +1748,10 @@ public class Epoch implements gsfc.nssdc.cdf.CDFConstants {
 	components[1] = j;
 	components[2] = k;
 	
-	components[3] = (long) ((double) hour_AD   % (double) 24.0);
-	components[4] = (long) ((double) minute_AD % (double) 60.0);
-	components[5] = (long) ((double) second_AD % (double) 60.0);
-	components[6] = (long) ((double) msec_AD   % (double) 1000.0);
+	components[3] = (long) (hour_AD   % (double) 24.0);
+	components[4] = (long) (minute_AD % (double) 60.0);
+	components[5] = (long) (second_AD % (double) 60.0);
+	components[6] = (long) (msec_AD   % (double) 1000.0);
 	
 	return components;
     }
