@@ -1,4 +1,4 @@
-// $Id: Epoch16.java,v 1.3 2020/06/24 18:33:13 mhliu Exp $
+// $Id: Epoch16.java,v 1.3 2021/06/07 16:56:13 mhliu Exp $
 /*
  * Copyright 1996-2014 United States Government as represented by the
  * Administrator of the National Aeronautics and Space Administration.
@@ -11,6 +11,7 @@ import gsfc.nssdc.cdf.CDFException;
 import java.text.NumberFormat;
 import java.text.DecimalFormat;
 
+import java.lang.reflect.*;
 import java.util.*;
 import java.text.*;
 
@@ -510,7 +511,7 @@ public class Epoch16 implements gsfc.nssdc.cdf.CDFConstants {
      *      
      * @return A string representation of the epoch
      */     
-    public static String toEncode(Object epoch, int style)
+    public static String toEncode(double[] epoch, int style)
     {       
         if (style < 0 || style > 4) style = 4;
         if (style == 0) return encode(epoch);
@@ -522,8 +523,9 @@ public class Epoch16 implements gsfc.nssdc.cdf.CDFConstants {
 
     /**
      * 
-     * Converts an EPOCH value into a readable date/time string.
-     * It present the string to the following ISO 8601 form: 
+     * Converts an EPOCH object into a scalar or array of readable 
+     * date/time string.
+     * Each string is presented in the following ISO 8601 form: 
      *
      * <PRE>
      *
@@ -533,13 +535,102 @@ public class Epoch16 implements gsfc.nssdc.cdf.CDFConstants {
      *
      * </PRE>
      *
-     * @param epoch the epoch value
+     * @param epoch the epoch object
      *
-     * @return A string representation of the epoch
+     * @return A string or array of strings representation of the epoch
      */
-    public static String toEncode(Object epoch)
+    public static Object toEncode(Object epoch)
     {
-        return toEncode(epoch, 4);
+        if (epoch == null) return null;
+        int nDims = CDFUtils.getNumDimensions(epoch);
+        long[] dimSizes = CDFUtils.getDimensionSizes(epoch);
+        if (nDims > 1) {
+          Object encodedArray = null;
+          String className = "java.lang.String";
+          int[] tempSizes = new int[nDims];
+          for (int j = 0; j < nDims; ++j) tempSizes[j] = (int) dimSizes[j];
+          Class arrayClass;
+          try {
+            arrayClass = Class.forName(className);
+            encodedArray = Array.newInstance(arrayClass, tempSizes);
+          } catch (ClassNotFoundException e) {
+            System.out.println("**** error BAD_ARGUMENT");
+            System.exit(0);
+          }
+          int arrayLength = ((Object[])epoch).length;
+          int i;
+          Object subObj;
+          for (i = 0; i < arrayLength; ++i) {
+            ((Object[])encodedArray)[i] = toEncode (((Object[])epoch)[i]);
+          }
+          return encodedArray;
+        } else {
+           long num = dimSizes[0] / 2;
+          return toEncode(epoch, num);
+        }
+
+    }
+
+    /**
+     * 
+     * Converts an EPOCH object into a scalar or array of readable 
+     * date/time string.
+     * Each string is presented in the following ISO 8601 form: 
+     *
+     * <PRE>
+     *
+     * Format:    yyyymmddThh:mm:ss.cccuuunnnppp
+     * Examples:  19900401T03:05:02.000111222333
+     *            19931010T23:45:49.999999999999
+     *
+     * </PRE>
+     *
+     * @param epoch the epoch object
+     *
+     * @return A string or array of strings representation of the epoch
+     */
+    public static String toEncode(double[] epoch)
+    {
+         return toEncode(epoch, 4);
+
+    }
+
+    /**
+     * 
+     * Converts an EPOCH object into a scalar or array(s) of readable 
+     * date/time string.
+     * Each string is  presented in the following ISO 8601 form: 
+     *
+     * <PRE>
+     *
+     * Format:    yyyymmddThh:mm:ss.cccuuunnnppp
+     * Examples:  19900401T03:05:02.000111222333
+     *            19931010T23:45:49.999999999999
+     *
+     * </PRE>
+     *
+     * @param epoch the epoch object
+     * @param size the number of CDF_EPOCH16 data
+     *
+     * @return A string or array(s) of strings representation of the epoch
+     */
+    public static Object toEncode(Object epoch, long size)
+    {
+        if (size == 1)
+          return toEncode((double[])epoch, 4);
+        else if (size == 0)
+          return null;
+        else {
+          int i;
+          String[] buff = new String[(int)size];
+          double[] ep16 = new double[2];
+          for (i = 0; i < size; ++i) {
+            ep16[0] = ((double[])epoch)[2*i];
+            ep16[1] = ((double[])epoch)[2*i+1];
+            buff[i] = toEncode(ep16, 4);
+          }
+          return buff; 
+        }
     }
 
     /**
@@ -556,7 +647,7 @@ public class Epoch16 implements gsfc.nssdc.cdf.CDFConstants {
      *
      * @return A unix time value
      */
-    public static double Epoch16toUnixTime (Object epoch)
+    public static double Epoch16toUnixTime (double[] epoch)
     {
         double seconds = ((double[])epoch)[0];
         double subsecs = ((double[])epoch)[1];
@@ -974,7 +1065,6 @@ public class Epoch16 implements gsfc.nssdc.cdf.CDFConstants {
 	String ePartStr = ePart.toString();
 	char sp = new DecimalFormat().getDecimalFormatSymbols().getDecimalSeparator();
 	appendPart(encoded,
-//		   ePartStr.substring(ePartStr.indexOf(".")+1),
 		   ePartStr.substring(ePartStr.indexOf(sp)+1),		   
 		   width,false);
     }
@@ -1158,8 +1248,7 @@ public class Epoch16 implements gsfc.nssdc.cdf.CDFConstants {
     {
 	long [] components = new long[10];
 	long jd,i,j,k,l,n;
-	double msec;
-        long second_AD, minute_AD, hour_AD, day_AD;
+	double msec, second_AD, minute_AD, hour_AD, day_AD;
 	double[] epoch16 = new double[2];
 	
 	epoch16[0] = ((double[]) epoch)[0];
@@ -1193,12 +1282,12 @@ public class Epoch16 implements gsfc.nssdc.cdf.CDFConstants {
                                          (MAX_EPOCH16_2_BINARY+1.0) :
                                          epoch16[1]);
 
-	second_AD = (long) epoch16[0];
-	minute_AD = second_AD / 60;
-	hour_AD = minute_AD / 60;
-	day_AD = hour_AD / 24;
+	second_AD = epoch16[0];
+	minute_AD = second_AD / 60.0;
+	hour_AD = minute_AD / 60.0;
+	day_AD = hour_AD / 24.0;
 	
-	jd = (long) (1721060 + day_AD);
+	jd = 1721060 + (long) day_AD;
 	l=jd+68569;
 	n=4*l/146097;
 	l=l-(146097*n+3)/4;
@@ -1214,9 +1303,9 @@ public class Epoch16 implements gsfc.nssdc.cdf.CDFConstants {
 	components[1] = j;
 	components[2] = k;
 	
-	components[3] = (long) ((double) hour_AD   % (double) 24.0);
-	components[4] = (long) ((double) minute_AD % (double) 60.0);
-	components[5] = (long) ((double) second_AD % (double) 60.0);
+	components[3] = (long) (hour_AD   % (double) 24.0);
+	components[4] = (long) (minute_AD % (double) 60.0);
+	components[5] = (long) (second_AD % (double) 60.0);
 
 	msec = epoch16[1];
         components[9] = (long) (msec % (double) 1000.0);
